/* Copyright (c) 2006 Arthur Huillet

GPL License */

#include "all_inc.h"


int game_running = 1; /*global "end of game" request flag*/
int cyclecount = 0; /*count of game cycles since the beginning, used for internal timing*/
int run_game_cycle = 1; /*incremented every 1/FPS seconds*/
int page_flip = 1; /*do page flipping ?*/

void increment_run_game_cycle()
{
run_game_cycle ++;
}

int usage()
{
fprintf(stderr, "Commandline options available :\n\tplease take a look at the sourcecode, file main.c\n");
return(0);
}

int main(int argc, char ** argv)
{
set_uformat(U_ASCII);
allegro_init();
install_keyboard();
install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL);
install_mouse();

init_logfac();
open_newlog("game_log.txt");
message_hdl ruletoadd;
	
/* first rule handles common end-user information, on-screen logging*/
ruletoadd.filedest=0;
ruletoadd.debuglvl=1;
ruletoadd.log_action=LOGTO_STDOUT;
add_rule(ruletoadd);

/* second rule handles more precise information, logging in file game_log.txt*/
ruletoadd.filedest=0;
ruletoadd.debuglvl=2;
ruletoadd.log_action=LOGTO_FILE;
add_rule(ruletoadd);

/*the third rule is only added if --debuglevel is specified */
ruletoadd.filedest=0;
ruletoadd.debuglvl=3;
ruletoadd.log_action=LOGTO_STDOUT;

log_msg(1, "Using log file %s", "game_log.txt");
log_msg(1, "Using %s", allegro_id);
log_msg(1, "Welcome to my TINS'06 entry !");

log_msg(2, "Setting up signal handler");
signal(SIGABRT, signal_handler);
signal(SIGFPE, signal_handler);
signal(SIGILL, signal_handler);
signal(SIGSEGV, signal_handler);
signal(SIGTERM, signal_handler);
signal(SIGINT, signal_handler);

switch(os_type)
	{
	case OSTYPE_WIN95:
	case OSTYPE_WIN98:
	case OSTYPE_WINME:
        case OSTYPE_WINNT:
        case OSTYPE_WIN2000:
        case OSTYPE_WINXP:
		log_msg(0, "Detected Windows OS, disabling page flipping !");
		page_flip = 0;
	}
log_msg(2, "Parsing commandline");
    int option;
    int longindex;
    //int page_flip = 1;
    struct option longopts[] = {

	{"no-page-flip", 		0, 				&page_flip, 			0},
	{"do-page-flip", 		0, 				&page_flip, 			1},
	{"help", 				0, 				NULL, 					'h'},
	{"debuglevel", 1, NULL, 'l'},
	{"select-gfx-mode", 1, NULL, 's'},
	{NULL, 0, NULL, 0},
    };


    char *optstring = "hl:";

    while ((option =
	    getopt_long(argc, argv, optstring, longopts,
			&longindex)) != -1) {

	switch (option) {

	case 'h':
	    usage();
	    exit(5);
	    break;
	case 'l':
	    ruletoadd.debuglvl = atoi(optarg);
	    add_rule(ruletoadd);
	    break;
	case '?':
		usage();
		exit(5);
	    break;
	}
    }


log_msg(1, "Starting game");
log_msg(2, "Requesting graphics mode");
RESOL_X = 800;
RESOL_Y = 600;
set_color_depth(16);
#ifndef _WIN32
if(set_gfx_mode(GFX_AUTODETECT_WINDOWED, RESOL_X, RESOL_Y, 0, 2*RESOL_Y) != 0)
   	{
   	log_msg(2,"set_gfx_mode(GFX_AUTODETECT_WINDOWED, %i, %i, 0, 0) failed, using another driver.", RESOL_X, RESOL_Y);
   	if (set_gfx_mode(GFX_SAFE, RESOL_X, RESOL_Y, 0, 0) != 0) 
   	    {
   	    set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
   	    allegro_message("Error setting graphics mode\n%s\n", allegro_error);
   	    return 1;
   	    }
   	}
#else
if( set_gfx_mode(GFX_AUTODETECT, RESOL_X, RESOL_Y, 0, 0) != 0)
   	{
   	log_msg(2,"set_gfx_mode(GFX_AUTODETECT, %i, %i, 0, 0) failed, using another driver.", RESOL_X, RESOL_Y);
   	if (set_gfx_mode(GFX_SAFE, RESOL_X, RESOL_Y, 0, 0) != 0) 
   	    {
   	    set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
   	    allegro_message("Error setting graphics mode\n%s\n", allegro_error);
   	    return 1;
   	    }
   	}
#endif
log_msg(2, "Setting up subbitmaps");
if(page_flip)
	{
	vscreen[0] = create_video_bitmap(RESOL_X, RESOL_Y);
	vscreen[1] = create_video_bitmap(RESOL_X, RESOL_Y);
	if(!vscreen[0] || !vscreen[1])
		{
		log_msg(0, "Error creating video bitmaps for page flipping : %s", allegro_error);
		exit(255);
		}
	}
else
	{
	vscreen[1] = NULL;
	vscreen[0] = create_bitmap(RESOL_X, RESOL_Y);
	if(!vscreen[0])
		{
		log_msg(0, "Error creating backbuffer : %s", allegro_error);
		exit(255);
		}
	}
page_num = 0; /*first page always 0*/

log_msg(2, "Initializing data");
first_enemy = NULL;
first_bomb = NULL;
last_bomb = NULL;
first_garbage = NULL;
set_color_conversion(COLORCONV_KEEP_TRANS | COLORCONV_MOST);
bground = load_bitmap("bground.tga", NULL);
enemy_spr = load_bitmap("enemy.bmp", NULL);
bomb_spr = load_bitmap("bomb.bmp", NULL);
bombexp_spr = load_bitmap("explosion.bmp", NULL);
log_msg(3, "Depths : bground %i, enemy %i, bomb %i, explosion %i", bitmap_color_depth(bground), bitmap_color_depth(enemy_spr), bitmap_color_depth(bomb_spr), bitmap_color_depth(bombexp_spr));
srand(time(NULL));
init_player();
init_railroad();
log_msg(2, "Setting up game timer");
GAME_PAUSED = 0;
LOCK_FUNCTION(increment_run_game_cycle);
LOCK_VARIABLE(run_game_cycle);
install_int_ex(increment_run_game_cycle, BPS_TO_TIMER(80));
while(game_running)
	{
	main_loop();
	if(cyclecount % 2)
		display_all();
	cyclecount ++;
	run_game_cycle --;
	while(!run_game_cycle)
		rest(1);
	}

log_msg(1, "Cleaning up memory");
free_railroad(railroad);
free_railroad(rail_pieces);
destroy_bitmap(vscreen[0]);
destroy_bitmap(player.sprite);
destroy_bitmap(train);
destroy_bitmap(part_sprites.dash);
destroy_bitmap(part_sprites.slash);
destroy_bitmap(part_sprites.antislash);
destroy_bitmap(part_sprites.pipe);
destroy_bitmap(bground);
destroy_bitmap(enemy_spr);
destroy_bitmap(bomb_spr);
destroy_bitmap(bombexp_spr);
free_enemies(first_enemy);
free_bombs(first_bomb);
free_garbage(first_garbage);
if(page_flip) destroy_bitmap(vscreen[1]);
exit_logfac();
exit(0);
}
END_OF_MAIN()
